{
open Lexing
let incr_linenum lexbuf =
    let pos = lexbuf.lex_curr_p in
    lexbuf.lex_curr_p <- { pos with
                           pos_lnum = pos.pos_lnum + 1;
                           pos_bol = pos.pos_cnum;
                         }

  let unexpected_char c lexbuf =
    Printf.printf "Char %s is unexpected at (line:%d, column:%d).\n" 
                  (Char.escaped c) 
                  (lexeme_start_p lexbuf).pos_lnum
                  ((lexeme_start_p lexbuf).pos_cnum - (lexeme_start_p lexbuf).pos_bol + 1)

type token =
  | ADD
  | MUL
  | LPAREN
  | RPAREN
  | INT of int
  | VAR of string
  | EOF

}

let alpha = ['a'-'z''A'-'Z']
let digit = ['0'-'9']

rule lex = parse
  | '+' { ADD }
  | '*' { MUL }
  | '(' { LPAREN }
  | ')' { RPAREN }
  | '-'?(digit|['1'-'9']digit+) as n { INT (int_of_string n) }
  | alpha(alpha|digit|'_')* as id { VAR id }
  | [' ''\t''\r']+ { lex lexbuf } (* ignore whitespace *)
  | '\n' { incr_linenum lexbuf; lex lexbuf } (* ignore newlines, but increment line num *)
  | eof { EOF }
  | _ as c 
      { unexpected_char c lexbuf; exit 1 }

{

}
